function ibm = generateIBM(target,interference,fs,LC,numFreqChan,fRange,winLength,winShift)

% generate an ideal binary mask(s) for a given source and set of
% interference signals for any number of channels
%
% Usage:
%
% ibm =
% generateIBM(target,interference,fs,LC,numFreqChan,fRange,winLength,winShift)
%
% Input:
%   *target: column vector(s) of target signal (sample X channel)
%   *interference: column vector(s) of interfence signal, where interference
%       is considered the summation of all interferening sources (sample X channel)
%   fs: sampling frequency of signals 
%       (default: 16 kHz)
%   LC: the local threshold criteria for IBM in dB 
%       (default: 0 dB)
%   numFreqChan: number of frequency channels for gammatone filtering
%       (default: 128)
%   fRange: minimum and maximum values in Hz for gammatone center frequencies
%       (default: [50 fs/2])
%   winLength: window length for each time frame in seconds
%       (default: 0.02)
%   winShift: length of shift between frames in seconds
%       (default: 0.01)
%   * denotes required inputs
%
% Output:
%   ibm: the generated ideal binary mask(s) 
%   (frequency X time frame X channel)
%
% John Woodruff (woodruff.95@osu.edu)
% Primary code by ZZ Jin, adapted by DL Wang


% if arguments not provided, set to default values
if nargin < 3
    fs = 16000;
end
if nargin < 4
    LC = 0;
end
if nargin < 5
    numFreqChan = 128;
end
if nargin < 6
    fRange = [50 fs/2];
end
if nargin < 7
    winLength = .02;
end
if nargin < 8
    winShift = winLength/2;
end

gt = makeFilters(numFreqChan, fRange, fs);

% convert frame length and frame shift to number of samples
winLength = floor(winLength*fs);
winShift = floor(winShift*fs);

% ensure that target and interference signals are correct shape
target = reshape(target,length(target),min(size(target)));
interference = reshape(interference,length(interference),min(size(interference)));

if prod(size(target))~=prod(size(interference))
    disp('Target and interference signals must have same length and number of channels');
else
    numChan = size(target,2);
    numFrames = floor(length(target)/winShift);
    ibm = zeros(numFreqChan,numFrames,numChan);
    for chan = 1:numChan
        disp(['Computing IBM for channel ',num2str(chan)]);

        for fChan = 1:numFreqChan
            ibm(fChan,:,chan) = channelIBM(target(:,chan),interference(:,chan),LC,gt(fChan,:),winLength,winShift);
        end
    end
end
